{**********************************************************}
{                                                          }
{          BLUE CODE for Delphi                            }
{                                                          }
{          Copyright (c) 1996-97 BlueCodeX                 }
{                                                          }
{**********************************************************}

unit DemoExec;

{$B-,G+,I+,N+,P+,R-,S-,T-,W-,X+}
{$C MOVEABLE DEMANDLOAD DISCARDABLE}

interface

uses
  WinTypes;

type

  PDemoExecInfo = ^TDemoExecInfo;
  TDemoExecInfo = record
    Name: array[0..12] of Char;
    Dir:  array[0..8] of Char;
    Caption: array[0..255] of Char;
    Path: array[0..155] of Char;
  end;

function MakeDemoExecInfo(DemoDir, DemoName, DemoCaption: Integer): PDemoExecInfo;
procedure BlcDemoExec(ExecInfo: PDemoExecInfo);
procedure DestroyBlcDemo(ExecInfo: PDemoExecInfo);
function FindBlcDemo(ExecInfo: PDemoExecInfo): HWnd;
function ShowBlcDemo(Wnd: HWnd; ShowNormal: Boolean): Boolean;

implementation

uses
  SysUtils, WinProcs, Controls, Forms, Dialogs, DemConst,
  Messages;


function FindBlcDemo(ExecInfo: PDemoExecInfo): HWnd;
begin
  Result := 0;
  if ExecInfo <> nil then Result := FindWindow('TForm1', ExecInfo^.Caption);
end;

procedure DestroyBlcDemo(ExecInfo: PDemoExecInfo);
var
  Wnd: HWnd;
begin
  Wnd := FindBlcDemo(ExecInfo);
  if Wnd <> 0 then SendMessage(Wnd, WM_CLOSE, 0, 0);
end;

{ Das hier verwendete Verfahren besitzt keine Allgemeingltigkeit. }
function ShowBlcDemo(Wnd: HWnd; ShowNormal: Boolean): Boolean;
var
  Owner: HWnd;
begin
  Result := False;
  if Wnd <> 0 then
  begin
    Owner := GetWindow(Wnd, GW_OWNER);
    if ShowNormal then
      if IsIconic(Owner) then SendMessage(Owner, WM_SYSCOMMAND, SC_RESTORE, 0)
      else if IsIconic(Wnd) then ShowWindow(Wnd, SW_SHOWNORMAL)
      else SetActiveWindow(Wnd)
    { Der Einsatz von ShowWindow(Wnd, SW_MINIMIZE) minimiert bei Anwendungen
      mit mehreren Formularen nur ein Formular. }
    else SendMessage(Owner, WM_SYSCOMMAND, SC_MINIMIZE, 0);
    Result := True;
  end;
end;

function GetDemoExePath(ExecInfo: PDemoExecInfo): Boolean;
const
  FileFilter = 'BlueCode-Demo|';
var
  SearchDlg: TOpenDialog;
  I: Integer;
  P: PChar;
begin
  Result := False;
  with ExecInfo^ do
  begin

    StrPCopy(Path, ExtractFilePath(Application.ExeName));
    for I := 0 to 1 do
    begin
      P := StrRScan(Path, '\');
      if P <> nil then P[0] := #0;
    end;
    StrCat(Path, '\');
    StrCat(Path, Dir);
    StrCat(Path,'\');
    StrCat(Path, Name);

    while not FileExists(StrPas(Path)) do
    begin
      if MessageDlg(FmtLoadStr(SInvalidExePath, [StrPas(Path)]),
        mtError, mbYesNoCancel, 0) <> mrYes then Exit;
      SearchDlg := TOpenDialog.Create(Application);

      try
        with SearchDlg do
        begin
          Filter := 'BlueCode-Demo|' + ExtractFileName(StrPas(Path));
          if Execute then
            { Nur Demoprogramme akzeptieren }
            if CompareText(ExtractFileName(FileName),
              StrPas(ExecInfo^.Name)) = 0 then StrPCopy(Path, FileName)
            else begin
              MessageDlg(FmtLoadStr(SNotFound, [StrPas(ExecInfo^.Name)]),
                mtError, [mbOk], 0);
              Exit;
             end
          else Exit;
        end;

      finally
        SearchDlg.Free;
      end;

    end;
  end;
  Result := True;
end;

procedure BlcDemoExec(ExecInfo: PDemoExecInfo);
var
  Buffer: array[0..255] of Char;
  I: Word;
begin
  if (ExecInfo <> nil) and not ShowBlcDemo(FindBlcDemo(ExecInfo), True)
    and (FileExists(StrPas(ExecInfo^.Path)) or GetDemoExePath(ExecInfo)) then
  begin
    { Ohne Intro Starten }
    StrPCopy(StrECopy(Buffer, ExecInfo^.Path), ' ' + LoadStr(SNoIntro));
    I := WinExec(Buffer, SW_SHOWNORMAL);
    { Fehler bei der Ausfhrung }
    if I < 22 then
      MessageDlg(FmtLoadStr(SExecError, [ExtractFileName(StrPas(ExecInfo^.Path)),
       LoadStr(SFirstWinExecError + I)]), mtError, [mbOk], 0);
    end;
end;

function MakeDemoExecInfo(DemoDir, DemoName, DemoCaption: Integer): PDemoExecInfo;
begin
  GetMem(Result, SizeOf(TDemoExecInfo));
  with Result^ do
  begin
    StrPCopy(Name, LoadStr(DemoName));
    StrPCopy(Dir, LoadStr(DemoDir));
    StrPCopy(Caption, LoadStr(DemoCaption));
    if not GetDemoExePath(Result) then Path[0] := #0;
  end;
end;

end.
